/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

#include <stdio.h>
#include "system.h"
#include "altera_avalon_pio_regs.h"
#include "altera_avalon_performance_counter.h"
#include "alt_types.h"

// needed for NIOS ID macro
#include "nios2.h"

// needed for loading data into a section 
#include "sys/alt_load.h"


alt_u16 number_to_sevenseg (unsigned int digit);
void check_cpu_id();
void test_vector_unit();
void vector_allinstr_test();

int main()
{
  alt_u8 led = 0x01;
  alt_u8 dir = 1;
  volatile int i;
  unsigned int perf_counter_value;
  
  //PERF_RESET(PERFORMANCE_COUNTER_BASE);
  //PERF_START_MEASURING(PERFORMANCE_COUNTER_BASE);
     
  // Get our processor ID (vector processor is ID 3)
  check_cpu_id();
  
     
  /* 
   * Infinitely shift a variable with one bit set back and forth, and write
   * it to the LED PIO.  Software loop provides delay element.
   */
  while (1) 
  {
    if (led & 0x81) 
    {
      dir = (dir ^ 0x1);
    }

    if (dir) 
    {
      led = (led >> 1);
    } 
    else 
    {
      led = (led << 1);
    }
    IOWR_ALTERA_AVALON_PIO_DATA(LED_PIO_BASE, led);


    // Get our processor ID (vector processor is ID 3)
    //check_cpu_id();
    
    //test_vector_unit();
    vector_allinstr_test();
    
    
    /* UTIIe seems to hang on printf */
    //printf("blah!");
    
    /* UTIIe seems to hang when I try the performance counter */
    //PERF_BEGIN(PERFORMANCE_COUNTER_BASE,1);
    
    i = 0;
    while (i<50000)
      i++;

    //PERF_END(PERFORMANCE_COUNTER_BASE,1);
    //perf_counter_value = (unsigned int)perf_get_section_time(PERFORMANCE_COUNTER_BASE,1);
    //IOWR_ALTERA_AVALON_PIO_DATA(SEVEN_SEG_PIO_BASE, number_to_sevenseg(perf_counter_value));
    
  }

    //PERF_STOP_MEASURING(PERFORMANCE_COUNTER_BASE);

  return 0;
}


/* Prints a two-digit decimal number on the seven segment display */
alt_u16 number_to_sevenseg (unsigned int number) {
    
    alt_u16  ls_sevenseg_mask=0x8181, ms_sevenseg_mask=0x8181;
    unsigned int ms_digit=1, ls_digit=1;

    ms_digit = (number / 10) % 10;
    ls_digit = number % 10;
    
    switch (ls_digit) {
        case 0: ls_sevenseg_mask = 0xff81;
                break;
        case 1: ls_sevenseg_mask = 0xffcf;
                break;
        case 2: ls_sevenseg_mask = 0xff92;
                break;
        case 3: ls_sevenseg_mask = 0xff86;
                break;
        case 4: ls_sevenseg_mask = 0xffcc;
                break;
        case 5: ls_sevenseg_mask = 0xffa4;
                break;
        case 6: ls_sevenseg_mask = 0xffa0;
                break;
        case 7: ls_sevenseg_mask = 0xff8f;
                break;
        case 8: ls_sevenseg_mask = 0xff80;
                break;
        case 9: ls_sevenseg_mask = 0xff84;
                break;
        default:    ls_sevenseg_mask = 0xff00;
                break;
    }
    
    switch (ms_digit) {
        case 0: ms_sevenseg_mask = 0x81ff;
                break;
        case 1: ms_sevenseg_mask = 0xcfff;
                break;
        case 2: ms_sevenseg_mask = 0x92ff;
                break;
        case 3: ms_sevenseg_mask = 0x86ff;
                break;
        case 4: ms_sevenseg_mask = 0xccff;
                break;
        case 5: ms_sevenseg_mask = 0xa4ff;
                break;
        case 6: ms_sevenseg_mask = 0xa0ff;
                break;
        case 7: ms_sevenseg_mask = 0x8fff;
                break;
        case 8: ms_sevenseg_mask = 0x80ff;
                break;
        case 9: ms_sevenseg_mask = 0x84ff;
                break;
        default:    ms_sevenseg_mask = 0x00ff;
                break;
    }
    
    return  (ms_sevenseg_mask & ls_sevenseg_mask);
}


/* Function to check CPU ID of current processor */
void check_cpu_id() {
  unsigned int id;
  NIOS2_READ_CPUID(id);
  //id += 1;
  IOWR_ALTERA_AVALON_PIO_DATA(SEVEN_SEG_PIO_BASE, number_to_sevenseg(id));
}    


/* Simple test of vins.vs/vext.vs */
void test_vector_unit() {
  int insertnum = 2;
  int vindex = 1;
  int addvalue = 1;
  unsigned int extractnum;
  
  
  __asm__ ( "vmstc      vindex, %2 \n\t"   
            "vins.vs    v1, %3 \n\t"
            "vadd.vs    v1, v1, %1 \n\t"
            "vext.vs    %0, v1"
            : "=r" (extractnum)
            : "r" (addvalue), "r" (vindex), "0" (insertnum)
            );
  
            
  //__asm__ ("vsub    v3, v1, v2 \n\t");
  //__asm__ ("vadd.vs    v1, v1, r10 \n\t");
  
  IOWR_ALTERA_AVALON_PIO_DATA(SEVEN_SEG_PIO_BASE, number_to_sevenseg(extractnum));
}


/* Runs through all vector instructinos */
void vector_allinstr_test() {
    
    __asm__ ( 
    //--------------------------------------------------------
    // Setup scalar registers
    //--------------------------------------------------------
    "addi    r17, r0, 1\n\t"
    "addi    r18, r0, 2\n\t"
    "addi    r19, r0, 4\n\t"
    "addi    r20, r0, 0xC0000000\n\t"
    "addi    r21, r0, 16\n\t"
    
    //--------------------------------------------------------
    // Test suite 1: Signed arithmetic instructions
    //--------------------------------------------------------
    // change vector length
    "addi    r16, r0, 31\n\t"                 // set vector length
    "vmstc   VL, r16\n\t"
    
    "vadd.vv     v3, v1, v2\n\t"
    "vadd.vs.1   v4, v2, r17\n\t"     // r17=0x00000001
    "vsub.vv     v5, v1, v2\n\t"
    "vsub.vs.1   v6, v1, r17\n\t"     // r17=0x00000001
    "vsub.sv     v7, r18, v1\n\t"     // r18=0x00000002
    
    "vabs        v8, v7\n\t"
    "vabs.1      v9, v7\n\t"
    "vabsdiff.vv     v10, v1, v2\n\t"
    "vabsdiff.vv.1   v11, v1, v2\n\t" 
    "vabsdiff.vs     v12, v2, r17\n\t"
    
    "vmin.vv     v13, v1, v7\n\t"
    "vmin.vs.1   v14, v1, r19\n\t"        // r19=0x00000004
    "vmax.vv.1   v15, v1, v7\n\t"
    "vmax.vs     v16, v12, r19\n\t"   // r19=0x00000004
    
    "vmerge.vv   v17, v1, v7\n\t"
    "vmerge.vv.1 v18, v1, v7\n\t"
    "vmerge.vs.1 v19, v1, r17\n\t"
    "vmerge.sv.1 v20, r17, v1\n\t"
    
    
    //  vmulhi.vv       v0, v20, v21
    //  vmulhi.vv.1     v0, v20, v21
    //  vmulhi.sv       v0, r23, v21
    //  vmulhi.sv.1     v0, r23, v21
    //  vmulhiu.vv      v0, v20, v21
    //  vmulhiu.vv.1    v0, v20, v21
    //  vmulhiu.sv      v0, r23, v21
    //  vmulhiu.sv.1    v0, r23, v21
    //  vmullo.vv       v0, v20, v21
    //  vmullo.vv.1     v0, v20, v21
    //  vmullo.sv       v0, r23, v21
    //  vmullo.sv.1     v0, r23, v21
    //  vmullou.vv      v0, v20, v21
    //  vmullou.vv.1    v0, v20, v21
    //  vmullou.sv      v0, r23, v21
    //  vmullou.sv.1    v0, r23, v21
    
    //--------------------------------------------------------
    // Test suite 2: Logical instructions
    //--------------------------------------------------------
    // .set noat // turn off assembler optimization warnings
    // change vector length
    "addi    r16, r0, 24\n\t"                 // set vector length
    "vmstc   VL, r16\n\t"
    
    "vsll.vv.1   v3, v1, v2\n\t"
    "vsll.sv     v4, r17, v1\n\t"     // r17=0x00000001
    "vsll.vs     v5, v1, r18\n\t"     // r18=0x00000002
    
    "vsub.sv     v6, r20, v0\n\t"     // r20=0xc0000000
    "vsra.vv     v7, v2, v6\n\t"
    "vsra.sv.1   v8, r17, v6\n\t"     // r17=0x00000001
    "vsra.vs     v9, v2, r20\n\t"     // r20=0xC0000000
    "vsrl.vv     v10, v1, v2\n\t"
    "vsrl.sv.1   v11, r17, v1\n\t"        // r17=0x00000001
    "vsrl.vs     v12, v1, r20\n\t"    // r20=0xC0000000
    "vrot.vv.1   v13, v1, v2\n\t"
    "vrot.sv.1   v14, r17, v1\n\t"
    "vrot.vs     v15, v1, r18\n\t"        // r18=0x00000002
    
    "vand.vv.1   v16, v1, v0\n\t"
    "vand.vs     v17, v1, r17\n\t"
    "vand.vs.1   v18, v1, r0\n\t"     // r0=0x00000000
    "vor.vv.1    v19, v1, v8\n\t"
    "vor.vs      v20, v0, r17\n\t"
    "vxor.vv.1   v21, v1, v1\n\t"
    "vxor.vs     v22, v1, r17\n\t"
    
    
    //--------------------------------------------------------
    // Test suite 3: Vector comparison, flag logical instructions
    //--------------------------------------------------------
    // .set noat // turn off assembler optimization warnings
    // change vector length
    "addi    r16, r0, 17\n\t"                 // set vector length
    "vmstc   VL, r16\n\t"
    "vsub.vs     v5, v1, r17\n\t"
    
    // change vector length
    "addi    r16, r0, 8\n\t"                  // set vector length
    "vmstc   VL, r16\n\t"
    "vadd.vs     v5, v5, r17\n\t"
    
    // change vector length
    "addi    r16, r0, 17\n\t"                 // set vector length
    "vmstc   VL, r16\n\t"
    
    "vcmpe.vv        vfgr0, v1, v5\n\t"
    "vcmpe.vs.1      vfgr1, v1, r18\n\t"      // r18=0x00000002
    "vcmpne.vv.1     vfgr2, v1, v2\n\t"
    "vcmpne.vs.1     vfgr3, v1, r18\n\t"
    
    "vcmplt.vv       vfgr4, v1, v5\n\t"
    "vcmplt.sv       vfgr5, r21, v5\n\t"      // r21=0x00000010
    "vcmplt.vs.1     vfgr6, v5, r21\n\t"      // r21=0x00000010
    
    "vcmple.vv       vfgr7, v1, v5\n\t"
    "vcmple.sv       vfgr8, r21, v5\n\t"      // r21=0x00000010
    "vcmple.vs       vfgr9, v5, r21\n\t"      // r21=0x00000010
    
    // Vector flag logical
    //addi       r16, r0, //14
    //vmstc      VL, r16
    "vfand.vv        vfgr10, vfmask1, vfmask1\n\t"
    "vfor.vv         vfgr11, vfmask0, vfmask1\n\t"
    "vfxor.vv        vfgr12, vfmask1, vfmask1\n\t"
    "vfnor.vv        vfgr13, vfmask1, vfmask0\n\t"
    "vfand.vs        vfgr14, vfmask1, r0\n\t"
    "vfor.vs         vfgr15, vfmask1, r17\n\t"
    "vfxor.vs        vfgr16, vfmask1, r17\n\t"
    "vfnor.vs        vfgr17, vfmask1, r17\n\t"


    //--------------------------------------------------------
    // Test suite: Vector loads
    //--------------------------------------------------------
    // optional setup for in-system testing
    "movhi   r21, 1\n\t"
    "addi    r21, r0, 0x0010\n\t"
    "movhi   r22, 1\n\t"
    "addi    r22, r0, 0x0016\n\t"
    "movhi   r23, 1\n\t"
    "addi    r23, r0, 0x0019\n\t"
    
    // .set noat // turn off assembler optimization warnings
    // change vector length
    "addi    r16, r0, 17\n\t"                 // set vector length
    "vmstc   VL, r16\n\t"
    
    "ldw     r13, 0(r21)\n\t"     // addr=0x00000010
    "ldh     r14, 0(r22)\n\t"     // addr=0x00000016
    "ldb     r15, 0(r23)\n\t"     // addr=0x00000019
    
    "vld.w   v1, vbase1, vinc0\n\t"           // base=0x04, inc=0
    "vlds.w  v2, vbase2, vstride3, vinc0\n\t" // base=0x08, stride=3, inc=0
    "vld.h   v3, vbase5, vinc0\n\t"           // base=0x12, inc=0
    "vldsu.h v4, vbase8, vstride2, vinc0\n\t" // base=0x1A, stride=2, inc=0
    "vldu.b  v5, vbase9, vinc0\n\t"           // base=0x1B, inc=0
    "vlds.b  v6, vbase15, vstride5, vinc0\n\t"    // base=0x21, stride=5, inc=0
    "vlds.w  v7, vbase14, vstride7, vinc0\n\t"    // base=0x20, stride=20, inc=0
    
    
    // change vector length
    //addi   r5, r0, 17
    //vmstc  VL, r5
    
    // Shift offsets for indexed load
    "vsll.sv     v33, r2, v32\n\t"            // shift v32 by r2=1
    "vsll.sv     v34, r18, v32\n\t"           // shift v32 by r18=2
    
    // Indexed load
    "vldx.w      v8, v34, vbase2\n\t"         // base=0x00000008
    "vldx.w.1    v9, v34, vbase2\n\t"         // base=0x00000008
    "vldx.h      v10, v33, vbase5\n\t"        // base=0x00000012
    "vldx.h.1    v11, v33, vbase5\n\t"        // base=0x00000012
    "vldxu.h     v12, v33, vbase5\n\t"        // base=0x00000012
    "vldxu.h.1   v13, v33, vbase5\n\t"        // base=0x00000012
    "vldx.b      v14, v32, vbase13\n\t"       // base=0x0000001F
    "vldx.b.1    v15, v32, vbase13\n\t"       // base=0x0000001F
    "vldxu.b     v16, v32, vbase13\n\t"       // base=0x0000001F
    "vldxu.b.1   v17, v32, vbase13\n\t"       // base=0x0000001F
    
    // change vector length
    "addi    r16, r0, 28\n\t"
    "vmstc   VL, r16\n\t"
    
    // Vector flag load
    "vfld        vfgr0, vbase1, vinc0\n\t"    // base=0x00000004

    //--------------------------------------------------------
    // Test suite: Vector stores
    //--------------------------------------------------------
    // optional setup for in-system testing
    "movhi   r21, 1\n\t"
    "addi    r21, r0, 0x0010\n\t"
    "movhi   r22, 1\n\t"
    "addi    r22, r0, 0x0016\n\t"
    "movhi   r23, 1\n\t"
    "addi    r23, r0, 0x0019\n\t"
    
    // .set noat // turn off assembler optimization warnings
    // change vector length
    "addi    r16, r0, 9\n\t"                  // set vector length
    "vmstc   VL, r16\n\t"
    
    // Scalar stores
    "stw     r15, 0(r21)\n\t"     // data=0x00000001, addr=0x00000010
    "sth     r15, 0(r22)\n\t"     // data=0x00000001, addr=0x00000016
    "stb     r15, 0(r23)\n\t"     // data=0x00000001, addr=0x00000019
    
    
    // Okay to test them altogether if only analyzing trace
    // the instructions will overwrite each other's results
    "vst.w       v1, vbase1, vinc5\n\t"               // base=0x04, inc=0x20
    "vsts.w.1    v1, vbase1, vstride7, vinc0\n\t"     // base=0x24, stride=0x14
    "vst.h       v1, vbase3, vinc0\n\t"               // base=0x0c
    "vsts.h      v1, vbase5, vstride4, vinc0\n\t"     // base=0x12, stride=4
    "vst.b       v1, vbase13, vinc0\n\t"
    "vst.b.1     v1, vbase13, vinc0\n\t"              // base=0x1f
    "vsts.b      v1, vbase5, vstride3, vinc0\n\t"     // base=0x12, stride=3
    
    // change vector length
    "addi    r16, r0, 8\n\t"                  // set vector length
    "vmstc   VL, r16\n\t"
    "vst.w       v1, vbase1, vinc5\n\t"               // base=0x04, inc=0x20
    "vsts.w.1    v1, vbase1, vstride7, vinc0\n\t"     // base=0x24, stride=0x14
    
    // change vector length
    "addi    r16, r0, 7\n\t"                  // set vector length
    "vmstc   VL, r16\n\t"
    "vst.w       v1, vbase1, vinc5\n\t"               // base=0x04, inc=0x20
    "vsts.w.1    v1, vbase1, vstride7, vinc0\n\t"     // base=0x24, stride=0x14
    
    
    // change vector length
    //addi   r16, r0, 8
    //vmstc  VL, r16
    
    // shift offsets for indexed load
    "vsll.sv     v33, r2, v32\n\t"            // shift v32 by r2=1
    "vsll.sv     v34, r18, v32\n\t"           // shift v32 by r18=2
    
    // Indexed store
    "vstx.w      v1, v34, vbase3\n\t"     // base=0x0000000C
    "vstx.w.1    v1, v34, vbase14\n\t"    // base=0x00000020
    "vstx.h      v1, v33, vbase12\n\t"    // base=0x0000001E
    "vstx.h.1    v1, v33, vbase12\n\t"    // base=0x0000001E
    "vstx.b      v1, v32, vbase13\n\t"    // base=0x0000001F
    "vstx.b.1    v1, v32, vbase13\n\t"    // base=0x0000001F
    
    // Vector flag store
    "vfst        vfmask1, vbase4, vinc1\n\t"  // base=0x00000010, inc=4
    
    "addi    r16, r0, 8\n\t"
    "vmstc   VL, r16\n\t"
    "vfst        vfmask1, vbase4, vinc1\n\t"  // base=0x00000010, inc=4
    "addi    r16, r0, 17\n\t"
    "vmstc   VL, r16\n\t"
    "vfst        vfmask1, vbase4, vinc1\n\t"  // base=0x00000010, inc=4
    "addi    r16, r0, 24\n\t"
    "vmstc   VL, r16\n\t"
    "vfst        vfmask1, vbase4, vinc1\n\t"  // base=0x00000010, inc=4

    :   // inputs
    :   // outputs
    : "r13", "r14", "r15", "r16", "r17", "r18", "r19", "r20", "r21", "r22", "r23" // clobber list
    );   
}
